@@ -155,6 +155,9 @@ soup_message_headers_append (SoupMessageHeaders *hdrs,
 	SoupHeader header;
 	SoupHeaderSetter setter;
 
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (value != NULL);
+
 	header.name = intern_header_name (name, &setter);
 	header.value = g_strdup (value);
 	g_array_append_val (hdrs->array, header);
@@ -209,6 +212,8 @@ soup_message_headers_remove (SoupMessageHeaders *hdrs, const char *name)
 	SoupHeaderSetter setter;
 	int index;
 
+	g_return_if_fail (name != NULL);
+
 	name = intern_header_name (name, &setter);
 	while ((index = find_header (hdr_array, name, 0)) != -1) {
 		g_free (hdr_array[index].value);
@@ -243,6 +248,8 @@ soup_message_headers_get (SoupMessageHeaders *hdrs, const char *name)
 	char *value;
 	int index, i;
 
+	g_return_val_if_fail (name != NULL, NULL);
+
 	name = intern_header_name (name, NULL);
 	if (hdrs->concat) {
 		value = g_hash_table_lookup (hdrs->concat, name);
@@ -508,6 +515,13 @@ soup_message_headers_get_encoding (SoupMessageHeaders *hdrs)
 			return hdrs->encoding;
 	}
 
+	/* Per RFC 2616 4.4, a response body that doesn't indicate its
+	 * encoding otherwise is terminated by connection close, and a
+	 * request that doesn't indicate otherwise has no body. Note
+	 * that SoupMessage calls soup_message_headers_set_encoding()
+	 * to override the response body default for our own
+	 * server-side messages.
+	 */
 	hdrs->encoding = (hdrs->type == SOUP_MESSAGE_HEADERS_RESPONSE) ?
 		SOUP_ENCODING_EOF : SOUP_ENCODING_NONE;
 	return hdrs->encoding;
@@ -681,6 +695,8 @@ soup_message_headers_set_expectations (SoupMessageHeaders *hdrs,
  * If @end is %-1 and @start is negative, then it represents a "suffix
  * range", referring to the last -@start bytes of the resource body.
  * (Eg, the last 500 bytes would be @start = %-500 and @end = %-1.)
+ *
+ * Since: 2.26
  **/
 
 static int
@@ -713,6 +729,8 @@ sort_ranges (gconstpointer a, gconstpointer b)
  * Return value: %TRUE if @hdrs contained a "Range" header containing
  * byte ranges which could be parsed, %FALSE otherwise (in which case
  * @range and @length will not be set).
+ *
+ * Since: 2.26
  **/
 gboolean
 soup_message_headers_get_ranges (SoupMessageHeaders  *hdrs,
@@ -795,6 +813,8 @@ soup_message_headers_get_ranges (SoupMessageHeaders  *hdrs,
  * @ranges: an array of #SoupRange
  *
  * Frees the array of ranges returned from soup_message_headers_get_ranges().
+ *
+ * Since: 2.26
  **/
 void
 soup_message_headers_free_ranges (SoupMessageHeaders  *hdrs,
@@ -812,6 +832,8 @@ soup_message_headers_free_ranges (SoupMessageHeaders  *hdrs,
  * Sets @hdrs's Range header to request the indicated ranges. (If you
  * only want to request a single range, you can use
  * soup_message_headers_set_range().)
+ *
+ * Since: 2.26
  **/
 void
 soup_message_headers_set_ranges (SoupMessageHeaders  *hdrs,
@@ -852,6 +874,8 @@ soup_message_headers_set_ranges (SoupMessageHeaders  *hdrs,
  *
  * If you need to request multiple ranges, use
  * soup_message_headers_set_ranges().
+ *
+ * Since: 2.26
  **/
 void
 soup_message_headers_set_range (SoupMessageHeaders  *hdrs,
@@ -879,6 +903,8 @@ soup_message_headers_set_range (SoupMessageHeaders  *hdrs,
  *
  * Return value: %TRUE if @hdrs contained a "Content-Range" header
  * containing a byte range which could be parsed, %FALSE otherwise.
+ *
+ * Since: 2.26
  **/
 gboolean
 soup_message_headers_get_content_range (SoupMessageHeaders  *hdrs,
@@ -927,6 +953,8 @@ soup_message_headers_get_content_range (SoupMessageHeaders  *hdrs,
  * Sets @hdrs's Content-Range header according to the given values.
  * (Note that @total_length is the total length of the entire resource
  * that this is a range of, not simply @end - @start + 1.)
+ *
+ * Since: 2.26
  **/
 void
 soup_message_headers_set_content_range (SoupMessageHeaders  *hdrs,
@@ -1029,16 +1057,18 @@ content_type_setter (SoupMessageHeaders *hdrs, const char *value)
  * its value in *@content_type and *@params. @params can be %NULL if you
  * are only interested in the content type itself.
  *
- * Return value: %TRUE if @hdrs contains a "Content-Type" header,
- * %FALSE if not (in which case *@content_type and *@params will be
- * unchanged).
+ * Return value: a string with the value of the "Content-Type" header
+ * or NULL if @hdrs does not contain that header (in which case
+ * *@params will be * unchanged, is it has been given).
+ *
+ * Since: 2.26
  **/
 const char *
 soup_message_headers_get_content_type (SoupMessageHeaders  *hdrs,
 				       GHashTable         **params)
 {
 	if (!hdrs->content_type)
-		return FALSE;
+		return NULL;
 
 	if (params)
 		parse_content_foo (hdrs, "Content-Type", NULL, params);
@@ -1053,6 +1083,8 @@ soup_message_headers_get_content_type (SoupMessageHeaders  *hdrs,
  *
  * Sets the "Content-Type" header in @hdrs to @content_type,
  * optionally with additional parameters specified in @params.
+ *
+ * Since: 2.26
  **/
 void
 soup_message_headers_set_content_type (SoupMessageHeaders  *hdrs,
@@ -1089,6 +1121,8 @@ soup_message_headers_set_content_type (SoupMessageHeaders  *hdrs,
  * Return value: %TRUE if @hdrs contains a "Content-Disposition"
  * header, %FALSE if not (in which case *@disposition and *@params
  * will be unchanged).
+ *
+ * Since: 2.26
  **/
 gboolean
 soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
@@ -1125,6 +1159,8 @@ soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
  *
  * See soup_message_headers_get_content_disposition() for a discussion
  * of how Content-Disposition is used in HTTP.
+ *
+ * Since: 2.26
  **/
 void
 soup_message_headers_set_content_disposition (SoupMessageHeaders  *hdrs,
