@@ -8,6 +8,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "soup-address.h"
 #include "soup-auth.h"
 #include "soup-enum-types.h"
 #include "soup-marshal.h"
@@ -51,6 +52,17 @@
  *
  * Represents an HTTP message being sent or received.
  *
+ * @status_code will normally be a #SoupKnownStatusCode, eg,
+ * %SOUP_STATUS_OK, though of course it might actually be an unknown
+ * status code. @reason_phrase is the actual text returned from the
+ * server, which may or may not correspond to the "standard"
+ * description of @status_code. At any rate, it is almost certainly
+ * not localized, and not very descriptive even if it is in the user's
+ * language; you should not use @reason_phrase in user-visible
+ * messages. Rather, you should look at @status_code, and determine an
+ * end-user-appropriate message based on that and on what you were
+ * trying to do.
+ *
  * As described in the #SoupMessageBody documentation, the
  * @request_body and @response_body %data fields will not necessarily
  * be filled in at all times. When they are filled in, they will be
@@ -62,11 +74,16 @@
  * filled in right before libsoup writes the request to the network,
  * but you should not count on this; use soup_message_body_flatten()
  * if you want to ensure that %data is filled in. @response_body's
- * %data will be filled in before #SoupMessage::finished is emitted,
- * unless you set the %SOUP_MESSAGE_OVERWRITE_CHUNKS flag.
+ * %data will be filled in before #SoupMessage::finished is emitted.
  *
  * For a server-side #SoupMessage, @request_body's %data will be
  * filled in before #SoupMessage::got_body is emitted.
+ *
+ * To prevent the %data field from being filled in at all (eg, if you
+ * are handling the data from a #SoupMessage::got_chunk, and so don't
+ * need to see it all at the end), call
+ * soup_message_body_set_accumulate() on @response_body or
+ * @request_body as appropriate, passing %FALSE.
  **/
 
 G_DEFINE_TYPE (SoupMessage, soup_message, G_TYPE_OBJECT)
@@ -75,6 +92,7 @@ enum {
 	WROTE_INFORMATIONAL,
 	WROTE_HEADERS,
 	WROTE_CHUNK,
+	WROTE_BODY_DATA,
 	WROTE_BODY,
 
 	GOT_INFORMATIONAL,
@@ -97,6 +115,7 @@ enum {
 	PROP_URI,
 	PROP_HTTP_VERSION,
 	PROP_FLAGS,
+	PROP_SERVER_SIDE,
 	PROP_STATUS_CODE,
 	PROP_REASON_PHRASE,
 
@@ -118,7 +137,7 @@ soup_message_init (SoupMessage *msg)
 	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
 
 	priv->io_status = SOUP_MESSAGE_IO_STATUS_IDLE;
-	priv->http_version = SOUP_HTTP_1_1;
+	priv->http_version = priv->orig_http_version = SOUP_HTTP_1_1;
 
 	msg->request_body = soup_message_body_new ();
 	msg->request_headers = soup_message_headers_new (SOUP_MESSAGE_HEADERS_REQUEST);
@@ -138,6 +157,8 @@ finalize (GObject *object)
 
 	if (priv->uri)
 		soup_uri_free (priv->uri);
+	if (priv->addr)
+		g_object_unref (priv->addr);
 
 	if (priv->auth)
 		g_object_unref (priv->auth);
@@ -149,7 +170,7 @@ finalize (GObject *object)
 	soup_message_body_free (msg->response_body);
 	soup_message_headers_free (msg->response_headers);
 
-	g_free ((char *) msg->reason_phrase);
+	g_free (msg->reason_phrase);
 
 	G_OBJECT_CLASS (soup_message_parent_class)->finalize (object);
 }
@@ -212,6 +233,13 @@ soup_message_class_init (SoupMessageClass *message_class)
 	 * @msg: the message
 	 *
 	 * Emitted immediately after writing a body chunk for a message.
+	 *
+	 * Note that this signal is not parallel to
+	 * #SoupMessage::got_chunk; it is emitted only when a complete
+	 * chunk (added with soup_message_body_append() or
+	 * soup_message_body_append_buffer()) has been written. To get
+	 * more useful continuous progress information, use
+	 * #SoupMessage::wrote_body_data.
 	 **/
 	signals[WROTE_CHUNK] =
 		g_signal_new ("wrote_chunk",
@@ -222,6 +250,30 @@ soup_message_class_init (SoupMessageClass *message_class)
 			      soup_marshal_NONE__NONE,
 			      G_TYPE_NONE, 0);
 
+	/**
+	 * SoupMessage::wrote-body-data:
+	 * @msg: the message
+	 * @chunk: the data written
+	 *
+	 * Emitted immediately after writing a portion of the message
+	 * body to the network.
+	 *
+	 * Unlike #SoupMessage::wrote_chunk, this is emitted after
+	 * every successful write() call, not only after finishing a
+	 * complete "chunk".
+	 *
+	 * Since: 2.4.1
+	 **/
+	signals[WROTE_BODY_DATA] =
+		g_signal_new ("wrote_body_data",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      0, /* FIXME after next ABI break */
+			      NULL, NULL,
+			      soup_marshal_NONE__BOXED,
+			      G_TYPE_NONE, 1,
+			      SOUP_TYPE_BUFFER);
+
 	/**
 	 * SoupMessage::wrote-body:
 	 * @msg: the message
@@ -319,7 +371,13 @@ soup_message_class_init (SoupMessageClass *message_class)
 			      NULL, NULL,
 			      soup_marshal_NONE__BOXED,
 			      G_TYPE_NONE, 1,
-			      SOUP_TYPE_BUFFER);
+			      /* Use %G_SIGNAL_TYPE_STATIC_SCOPE so that
+			       * the %SOUP_MEMORY_TEMPORARY buffers used
+			       * by soup-message-io.c when emitting this
+			       * signal don't get forcibly copied by
+			       * g_signal_emit().
+			       */
+			      SOUP_TYPE_BUFFER | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 	/**
 	 * SoupMessage::got-body:
@@ -379,6 +437,12 @@ soup_message_class_init (SoupMessageClass *message_class)
 			      G_TYPE_NONE, 0);
 
 	/* properties */
+	/**
+	 * SOUP_MESSAGE_METHOD:
+	 *
+	 * Alias for the #SoupMessage:method property. (The message's
+	 * HTTP method.)
+	 **/
 	g_object_class_install_property (
 		object_class, PROP_METHOD,
 		g_param_spec_string (SOUP_MESSAGE_METHOD,
@@ -386,6 +450,12 @@ soup_message_class_init (SoupMessageClass *message_class)
 				     "The message's HTTP method",
 				     SOUP_METHOD_GET,
 				     G_PARAM_READWRITE));
+	/**
+	 * SOUP_MESSAGE_URI:
+	 *
+	 * Alias for the #SoupMessage:uri property. (The message's
+	 * #SoupURI.)
+	 **/
 	g_object_class_install_property (
 		object_class, PROP_URI,
 		g_param_spec_boxed (SOUP_MESSAGE_URI,
@@ -393,6 +463,12 @@ soup_message_class_init (SoupMessageClass *message_class)
 				    "The message's Request-URI",
 				    SOUP_TYPE_URI,
 				    G_PARAM_READWRITE));
+	/**
+	 * SOUP_MESSAGE_HTTP_VERSION:
+	 *
+	 * Alias for the #SoupMessage:http-version property. (The
+	 * message's #SoupHTTPVersion.)
+	 **/
 	g_object_class_install_property (
 		object_class, PROP_HTTP_VERSION,
 		g_param_spec_enum (SOUP_MESSAGE_HTTP_VERSION,
@@ -401,6 +477,12 @@ soup_message_class_init (SoupMessageClass *message_class)
 				   SOUP_TYPE_HTTP_VERSION,
 				   SOUP_HTTP_1_1,
 				   G_PARAM_READWRITE));
+	/**
+	 * SOUP_MESSAGE_FLAGS:
+	 *
+	 * Alias for the #SoupMessage:flags property. (The message's
+	 * #SoupMessageFlags.)
+	 **/
 	g_object_class_install_property (
 		object_class, PROP_FLAGS,
 		g_param_spec_flags (SOUP_MESSAGE_FLAGS,
@@ -409,6 +491,25 @@ soup_message_class_init (SoupMessageClass *message_class)
 				    SOUP_TYPE_MESSAGE_FLAGS,
 				    0,
 				    G_PARAM_READWRITE));
+	/**
+	 * SOUP_MESSAGE_SERVER_SIDE:
+	 *
+	 * Alias for the #SoupMessage:server-side property. (%TRUE if
+	 * the message was created by #SoupServer.)
+	 **/
+	g_object_class_install_property (
+		object_class, PROP_SERVER_SIDE,
+		g_param_spec_boolean (SOUP_MESSAGE_SERVER_SIDE,
+				      "Server-side",
+				      "Whether or not the message is server-side rather than client-side",
+				      FALSE,
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * SOUP_MESSAGE_STATUS_CODE:
+	 *
+	 * Alias for the #SoupMessage:status-code property. (The
+	 * message's HTTP response status code.)
+	 **/
 	g_object_class_install_property (
 		object_class, PROP_STATUS_CODE,
 		g_param_spec_uint (SOUP_MESSAGE_STATUS_CODE,
@@ -416,6 +517,12 @@ soup_message_class_init (SoupMessageClass *message_class)
 				   "The HTTP response status code",
 				   0, 599, 0,
 				   G_PARAM_READWRITE));
+	/**
+	 * SOUP_MESSAGE_REASON_PHRASE:
+	 *
+	 * Alias for the #SoupMessage:reason-phrase property. (The
+	 * message's HTTP response reason phrase.)
+	 **/
 	g_object_class_install_property (
 		object_class, PROP_REASON_PHRASE,
 		g_param_spec_string (SOUP_MESSAGE_REASON_PHRASE,
@@ -430,6 +537,7 @@ set_property (GObject *object, guint prop_id,
 	      const GValue *value, GParamSpec *pspec)
 {
 	SoupMessage *msg = SOUP_MESSAGE (object);
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
 
 	switch (prop_id) {
 	case PROP_METHOD:
@@ -444,6 +552,13 @@ set_property (GObject *object, guint prop_id,
 	case PROP_FLAGS:
 		soup_message_set_flags (msg, g_value_get_flags (value));
 		break;
+	case PROP_SERVER_SIDE:
+		priv->server_side = g_value_get_boolean (value);
+		if (priv->server_side) {
+			soup_message_headers_set_encoding (msg->response_headers,
+							   SOUP_ENCODING_CONTENT_LENGTH);
+		}
+		break;
 	case PROP_STATUS_CODE:
 		soup_message_set_status (msg, g_value_get_uint (value));
 		break;
@@ -452,6 +567,7 @@ set_property (GObject *object, guint prop_id,
 					      g_value_get_string (value));
 		break;
 	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
@@ -476,6 +592,9 @@ get_property (GObject *object, guint prop_id,
 	case PROP_FLAGS:
 		g_value_set_flags (value, priv->msg_flags);
 		break;
+	case PROP_SERVER_SIDE:
+		g_value_set_boolean (value, priv->server_side);
+		break;
 	case PROP_STATUS_CODE:
 		g_value_set_uint (value, msg->status_code);
 		break;
@@ -483,6 +602,7 @@ get_property (GObject *object, guint prop_id,
 		g_value_set_string (value, msg->reason_phrase);
 		break;
 	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
@@ -643,6 +763,20 @@ soup_message_wrote_chunk (SoupMessage *msg)
 	g_signal_emit (msg, signals[WROTE_CHUNK], 0);
 }
 
+/**
+ * soup_message_wrote_body_data:
+ * @msg: a #SoupMessage
+ * @chunk: the data written
+ *
+ * Emits the %wrote_body_data signal, indicating that the IO layer
+ * finished writing a portion of @msg's body.
+ **/
+void
+soup_message_wrote_body_data (SoupMessage *msg, SoupBuffer *chunk)
+{
+	g_signal_emit (msg, signals[WROTE_BODY_DATA], 0, chunk);
+}
+
 /**
  * soup_message_wrote_body:
  * @msg: a #SoupMessage
@@ -700,15 +834,13 @@ static void
 got_body (SoupMessage *req)
 {
 	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (req);
+	SoupMessageBody *body;
 
-	if (!(priv->msg_flags & SOUP_MESSAGE_OVERWRITE_CHUNKS)) {
+	body = priv->server_side ? req->request_body : req->response_body;
+	if (soup_message_body_get_accumulate (body)) {
 		SoupBuffer *buffer;
 
-		/* Figure out *which* body we read, and flatten it. */
-		if (req->status_code == 0)
-			buffer = soup_message_body_flatten (req->request_body);
-		else
-			buffer = soup_message_body_flatten (req->response_body);
+		buffer = soup_message_body_flatten (body);
 		soup_buffer_free (buffer);
 	}
 }
@@ -786,13 +918,7 @@ header_handler_metamarshal (GClosure *closure, GValue *return_value,
 	if (priv->io_status != SOUP_MESSAGE_IO_STATUS_RUNNING)
 		return;
 
-	/* If status_code is SOUP_STATUS_NONE, we're still processing
-	 * the request side; if it's not, we're processing the
-	 * response side.
-	 */
-	hdrs = (msg->status_code == SOUP_STATUS_NONE) ?
-		msg->request_headers : msg->response_headers;
-
+	hdrs = priv->server_side ? msg->request_headers : msg->response_headers;
 	if (soup_message_headers_get (hdrs, header_name)) {
 		closure->marshal (closure, return_value, n_param_values,
 				  param_values, invocation_hint,
@@ -1030,27 +1156,34 @@ soup_message_get_proxy_auth (SoupMessage *msg)
 void
 soup_message_cleanup_response (SoupMessage *req)
 {
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (req);
+
 	soup_message_body_truncate (req->response_body);
 	soup_message_headers_clear (req->response_headers);
+	if (priv->server_side) {
+		soup_message_headers_set_encoding (req->response_headers,
+						   SOUP_ENCODING_CONTENT_LENGTH);
+	}
 
 	req->status_code = SOUP_STATUS_NONE;
 	if (req->reason_phrase) {
-		g_free ((char *) req->reason_phrase);
+		g_free (req->reason_phrase);
 		req->reason_phrase = NULL;
 	}
+	priv->http_version = priv->orig_http_version;
+
 	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_STATUS_CODE);
 	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_REASON_PHRASE);
+	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_HTTP_VERSION);
 }
 
 /**
  * SoupMessageFlags:
  * @SOUP_MESSAGE_NO_REDIRECT: The session should not follow redirect
  * (3xx) responses received by this message.
- * @SOUP_MESSAGE_OVERWRITE_CHUNKS: Each chunk of the response will be
- * freed after its corresponding %got_chunk signal is emitted, meaning
- * %response will still be empty after the message is complete. You
- * can use this to save memory if you expect the response to be large
- * and you are able to process it a chunk at a time.
+ * @SOUP_MESSAGE_OVERWRITE_CHUNKS: Deprecated: equivalent to calling
+ * soup_message_body_set_accumulate() on the incoming message body
+ * (ie, %response_body for a client-side request), passing %FALSE.
  *
  * Various flags that can be set on a #SoupMessage to alter its
  * behavior.
@@ -1066,9 +1199,18 @@ soup_message_cleanup_response (SoupMessage *req)
 void
 soup_message_set_flags (SoupMessage *msg, SoupMessageFlags flags)
 {
-	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+	SoupMessagePrivate *priv;
 
-	SOUP_MESSAGE_GET_PRIVATE (msg)->msg_flags = flags;
+	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+
+	if ((priv->msg_flags ^ flags) & SOUP_MESSAGE_OVERWRITE_CHUNKS) {
+		soup_message_body_set_accumulate (
+			priv->server_side ? msg->request_body : msg->response_body,
+			!(flags & SOUP_MESSAGE_OVERWRITE_CHUNKS));
+	}
+
+	priv->msg_flags = flags;
 	g_object_notify (G_OBJECT (msg), SOUP_MESSAGE_FLAGS);
 }
 
@@ -1108,9 +1250,14 @@ soup_message_get_flags (SoupMessage *msg)
 void
 soup_message_set_http_version (SoupMessage *msg, SoupHTTPVersion version)
 {
-	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+	SoupMessagePrivate *priv;
 
-	SOUP_MESSAGE_GET_PRIVATE (msg)->http_version = version;
+	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+
+	priv->http_version = version;
+	if (msg->status_code == SOUP_STATUS_NONE)
+		priv->orig_http_version = version;
 	g_object_notify (G_OBJECT (msg), SOUP_MESSAGE_HTTP_VERSION);
 }
 
@@ -1153,6 +1300,10 @@ soup_message_is_keepalive (SoupMessage *msg)
 	    msg->method == SOUP_METHOD_CONNECT)
 		return TRUE;
 
+	/* Not persistent if the server sent a terminate-by-EOF response */
+	if (soup_message_headers_get_encoding (msg->response_headers) == SOUP_ENCODING_EOF)
+		return FALSE;
+
 	if (SOUP_MESSAGE_GET_PRIVATE (msg)->http_version == SOUP_HTTP_1_0) {
 		/* Only persistent if the client requested keepalive
 		 * and the server agreed.
@@ -1160,11 +1311,9 @@ soup_message_is_keepalive (SoupMessage *msg)
 
 		if (!c_conn || !s_conn)
 			return FALSE;
-		if (soup_header_contains (c_conn, "Keep-Alive") ||
-		    soup_header_contains (s_conn, "Keep-Alive"))
+		if (!soup_header_contains (c_conn, "Keep-Alive") ||
+		    !soup_header_contains (s_conn, "Keep-Alive"))
 			return FALSE;
-
-		return TRUE;
 	} else {
 		/* Normally persistent unless either side requested otherwise */
 		if (c_conn && soup_header_contains (c_conn, "close"))
@@ -1172,12 +1321,10 @@ soup_message_is_keepalive (SoupMessage *msg)
 		if (s_conn && soup_header_contains (s_conn, "close"))
 			return FALSE;
 
-		/* But not if the server sent a terminate-by-EOF response */
-		if (soup_message_headers_get_encoding (msg->response_headers) == SOUP_ENCODING_EOF)
-			return FALSE;
-
 		return TRUE;
 	}
+
+	return TRUE;
 }
 
 /**
@@ -1199,6 +1346,10 @@ soup_message_set_uri (SoupMessage *msg, SoupURI *uri)
 
 	if (priv->uri)
 		soup_uri_free (priv->uri);
+	if (priv->addr) {
+		g_object_unref (priv->addr);
+		priv->addr = NULL;
+	}
 	priv->uri = soup_uri_copy (uri);
 
 	g_object_notify (G_OBJECT (msg), SOUP_MESSAGE_URI);
@@ -1220,6 +1371,33 @@ soup_message_get_uri (SoupMessage *msg)
 	return SOUP_MESSAGE_GET_PRIVATE (msg)->uri;
 }
 
+/**
+ * soup_message_get_address:
+ * @msg: a #SoupMessage
+ *
+ * Gets the address @msg's URI points to. After first setting the
+ * URI on a message, this will be unresolved, although the message's
+ * session will resolve it before sending the message.
+ *
+ * Return value: the address @msg's URI points to
+ *
+ * Since: 2.26
+ **/
+SoupAddress *
+soup_message_get_address (SoupMessage *msg)
+{
+	SoupMessagePrivate *priv;
+
+	g_return_val_if_fail (SOUP_IS_MESSAGE (msg), NULL);
+
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	if (!priv->addr) {
+		priv->addr = soup_address_new (priv->uri->host,
+					       priv->uri->port);
+	}
+	return priv->addr;
+}
+
 /**
  * soup_message_set_status:
  * @msg: a #SoupMessage
@@ -1234,7 +1412,7 @@ soup_message_set_status (SoupMessage *msg, guint status_code)
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
 	g_return_if_fail (status_code != 0);
 
-	g_free ((char *) msg->reason_phrase);
+	g_free (msg->reason_phrase);
 
 	msg->status_code = status_code;
 	msg->reason_phrase = g_strdup (soup_status_get_phrase (status_code));
@@ -1259,7 +1437,7 @@ soup_message_set_status_full (SoupMessage *msg,
 	g_return_if_fail (status_code != 0);
 	g_return_if_fail (reason_phrase != NULL);
 
-	g_free ((char *) msg->reason_phrase);
+	g_free (msg->reason_phrase);
 
 	msg->status_code = status_code;
 	msg->reason_phrase = g_strdup (reason_phrase);
@@ -1325,23 +1503,22 @@ soup_message_get_io_status (SoupMessage *msg)
  * %length to indicate how much data it read.
  *
  * Generally, a custom chunk allocator would be used in conjunction
- * with %SOUP_MESSAGE_OVERWRITE_CHUNKS and #SoupMessage::got_chunk, as
- * part of a strategy to avoid unnecessary copying of data. However,
- * you cannot assume that every call to the allocator will be followed
- * by a call to your %got_chunk handler; if an I/O error occurs, then
- * the buffer will be unreffed without ever having been used. If your
- * buffer-allocation strategy requires special cleanup, use
- * soup_buffer_new_with_owner() rather than doing the cleanup from the
- * %got_chunk handler.
+ * with soup_message_body_set_accumulate() %FALSE and
+ * #SoupMessage::got_chunk, as part of a strategy to avoid unnecessary
+ * copying of data. However, you cannot assume that every call to the
+ * allocator will be followed by a call to your %got_chunk handler; if
+ * an I/O error occurs, then the buffer will be unreffed without ever
+ * having been used. If your buffer-allocation strategy requires
+ * special cleanup, use soup_buffer_new_with_owner() rather than doing
+ * the cleanup from the %got_chunk handler.
  *
- * The other thing to remember when using
- * %SOUP_MESSAGE_OVERWRITE_CHUNKS is that the buffer passed to the
- * %got_chunk handler will be unreffed after the handler returns, just
- * as it would be in the non-custom-allocated case. If you want to
- * hand the chunk data off to some other part of your program to use
- * later, you'll need to ref the #SoupBuffer (or its owner, in the
- * soup_buffer_new_with_owner() case) to ensure that the data remains
- * valid.
+ * The other thing to remember when using non-accumulating message
+ * bodies is that the buffer passed to the %got_chunk handler will be
+ * unreffed after the handler returns, just as it would be in the
+ * non-custom-allocated case. If you want to hand the chunk data off
+ * to some other part of your program to use later, you'll need to ref
+ * the #SoupBuffer (or its owner, in the soup_buffer_new_with_owner()
+ * case) to ensure that the data remains valid.
  **/
 void
 soup_message_set_chunk_allocator (SoupMessage *msg,
